import pglive.examples_pyqt5 as examples
import signal
from threading import Thread

import pyqtgraph as pg
from PyQt5.QtCore import QPointF
from PyQt5.QtWidgets import QWidget, QGridLayout, QLabel

from pglive.kwargs import Crosshair
from pglive.sources.data_connector import DataConnector
from pglive.sources.live_plot import LiveLinePlot
from pglive.sources.live_plot_widget import LivePlotWidget

"""
In this example, we create plot with Crosshair.
Crosshair has few signals implemented:
    sig_crosshair_moved - fired when crosshair is moved within plot area
    sig_crosshair_out - fired when crosshair leaves plot area
    sig_crosshair_in - fired when crosshair enters plot area
There are three text labels displayed below plot showing crosshair status, X and Y value.
"""

# Create parent widget
parent_widget = QWidget()
parent_layout = QGridLayout()
parent_widget.setLayout(parent_layout)

# Define crosshair parameters
kwargs = {Crosshair.ENABLED: True,
          Crosshair.LINE_PEN: pg.mkPen(color="red", width=1),
          Crosshair.TEXT_KWARGS: {"color": "green"}}

# Create plot widget
widget = LivePlotWidget(title="Line Plot and Crosshair @ 100Hz", **kwargs)
plot = LiveLinePlot()
widget.addItem(plot)

# Connect plot with DataConnector
data_connector = DataConnector(plot, max_points=600)

# Create crosshair X, Y label
ch_status_value = QLabel("Crosshair: Outside plot")
ch_x_value = QLabel("X: Unavailable")
ch_y_value = QLabel("Y: Unavailable")

# Add widgets int parent widget
parent_layout.addWidget(widget)
parent_layout.addWidget(ch_status_value)
parent_layout.addWidget(ch_x_value)
parent_layout.addWidget(ch_y_value)


def crosshair_moved(crosshair_pos: QPointF):
    """Update crosshair X, Y label when crosshair move"""
    ch_x_value.setText(f"X: {crosshair_pos.x()}")
    ch_y_value.setText(f"Y: {crosshair_pos.y()}")


def crosshair_out():
    """Update crosshair X, Y label when crosshair leaves plot area"""
    ch_status_value.setText("Crosshair: Outside plot")
    ch_x_value.setText(f"X: Unavailable")
    ch_y_value.setText(f"Y: Unavailable")


def crosshair_in():
    """Update crosshair X, Y label when crosshair enters plot area"""
    ch_status_value.setText("Crosshair: Inside plot")


# Connect moved and out signals with respective functions
widget.sig_crosshair_moved.connect(crosshair_moved)
widget.sig_crosshair_out.connect(crosshair_out)
widget.sig_crosshair_in.connect(crosshair_in)

parent_widget.show()

Thread(target=examples.sin_wave_generator, args=(data_connector,)).start()
signal.signal(signal.SIGINT, lambda sig, frame: examples.stop())
examples.app.exec()
examples.stop()



import pglive.examples_pyqt6 as examples
import signal
from math import sin
from threading import Thread
from time import sleep

from pglive.sources.data_connector import DataConnector
from pglive.sources.live_plot import LiveVBarPlot
from pglive.sources.live_plot_widget import LivePlotWidget

"""
In this example Vertical Bar plot is displayed.
Every update bar color is changed as well.
"""
win = LivePlotWidget(title="Coloured Vertical Bar Plot @ 100Hz")
plot = LiveVBarPlot(bar_width=1)
win.addItem(plot)

data_connector = DataConnector(plot, max_points=600)


def sin_wave_generator(*data_connectors):
    """Sin function generator, cycling colors of bar with each data point append"""
    x = 0
    while examples.running:
        x += 1
        for coloured_data_connector in data_connectors:
            color = next(examples.colors)
            coloured_data_connector.cb_append_data_point(sin(x * 0.01), x, pen=color, brush=color)
        sleep(0.01)


win.show()

Thread(target=sin_wave_generator, args=(data_connector,)).start()
signal.signal(signal.SIGINT, lambda sig, frame: examples.stop())
examples.app.exec()
examples.stop()



from typing import List, Union

import pyqtgraph as pg

from pglive.kwargs import LeadingLine

if pg.Qt.QT_LIB == pg.Qt.PYQT6:
    from PyQt6.QtCore import pyqtSlot, pyqtSignal, QPointF
    from PyQt6.QtGui import QPen
else:
    from PyQt5.QtCore import pyqtSlot, pyqtSignal, QPointF
    from PyQt5.QtGui import QPen


class MixinLivePlot:
    """Implements new_data slot for any plot"""

    @pyqtSlot(object, object, dict)
    def slot_new_data(self, y: List[Union[int, float]], x: List[Union[int, float]], kwargs) -> None:
        self.setData(x, y, **kwargs)


class MixinLiveBarPlot:
    """Implements new_data slot for Bar Plot"""
    sigPlotChanged = pyqtSignal()

    @pyqtSlot(object, object, dict)
    def slot_new_data(self, y: List[Union[int, float]], x: List[Union[int, float]], kwargs) -> None:
        self.setData(x, y, kwargs)


class MixinLeadingLine:
    """Implements leading line"""
    _hl_kwargs = None
    _vl_kwargs = None

    def set_leading_line(self, orientation: Union[LeadingLine.HORIZONTAL, LeadingLine.VERTICAL] = LeadingLine.VERTICAL,
                         pen: QPen = None, text_axis: str = LeadingLine.AXIS_X) -> dict:
        text_axis = text_axis.lower()
        assert text_axis in (LeadingLine.AXIS_X, LeadingLine.AXIS_Y)

        self.sigPlotChanged.connect(self.update_leading_line)

        if pen is None:
            pen = self.opts.get("pen")
        if orientation == LeadingLine.VERTICAL:
            _v_leading_line = pg.InfiniteLine(angle=90, movable=False, pen=pen)
            _v_leading_text = pg.TextItem(color="black", angle=-90, fill=pen.color())
            _v_leading_line.setZValue(999)
            _v_leading_text.setZValue(999)
            self._vl_kwargs = {"line": _v_leading_line, "text": _v_leading_text, "pen": pen, "text_axis": text_axis}
            return self._vl_kwargs
        elif orientation == LeadingLine.HORIZONTAL:
            _h_leading_line = pg.InfiniteLine(angle=0, movable=False, pen=pen)
            _h_leading_text = pg.TextItem(color="black", fill=pen.color())
            _h_leading_text.setZValue(999)
            _h_leading_text.setZValue(999)
            self._hl_kwargs = {"line": _h_leading_line, "text": _h_leading_text, "pen": pen, "text_axis": text_axis}
            return self._hl_kwargs

    @pyqtSlot()
    def update_leading_line(self):
        raise NotImplementedError

    def x_format(self, value: Union[int, float]) -> str:
        """X tick format"""
        return str(round(value, 4))

    def y_format(self, value: Union[int, float]) -> str:
        """Y tick format"""
        return str(round(value, 4))

    @pyqtSlot()
    def update_leading_text(self, x: float, y: float) -> None:
        """Update position and text of Vertical and Horizontal leading text"""
        vb = self.getViewBox()
        width, height = vb.width(), vb.height()

        if self._vl_kwargs is not None:
            text_axis = self.x_format(x) if self._vl_kwargs["text_axis"] == LeadingLine.AXIS_X else self.y_format(y)
            self._vl_kwargs["text"].setText(text_axis)
            pixel_pos = vb.mapViewToScene(QPointF(x, y))
            y_pos = 0 + self._vl_kwargs["text"].boundingRect().height() + 10
            new_pos = vb.mapSceneToView(QPointF(pixel_pos.x(), y_pos))
            self._vl_kwargs["text"].setPos(new_pos.x(), new_pos.y())

        if self._hl_kwargs is not None:
            text_axis = self.x_format(x) if self._hl_kwargs["text_axis"] == LeadingLine.AXIS_X else self.y_format(y)
            self._hl_kwargs["text"].setText(text_axis)
            pixel_pos = vb.mapViewToScene(QPointF(x, y))
            x_pos = width - self._hl_kwargs["text"].boundingRect().width() + 21
            new_pos = vb.mapSceneToView(QPointF(x_pos, pixel_pos.y()))
            self._hl_kwargs["text"].setPos(new_pos.x(), new_pos.y())



import pglive.examples_pyqt6 as examples
from threading import Thread

from pglive.sources.data_connector import DataConnector
from pglive.sources.live_plot import LiveHBarPlot
from pglive.sources.live_plot_widget import LivePlotWidget

"""
In this example Horizontal Bar plot is displayed.
"""
win = LivePlotWidget(title="Horizontal Bar Plot @ 100Hz")
plot = LiveHBarPlot(bar_height=1, brush="green", pen="green")
win.addItem(plot)

data_connector = DataConnector(plot, max_points=600)
win.show()

Thread(target=examples.sin_wave_generator, args=(data_connector,)).start()
examples.app.exec()
examples.stop()



import numpy as np
import pyqtgraph as pg
from pyqtgraph import GraphicsObject

from pglive.sources.live_mixins import MixinLivePlot, MixinLeadingLine, MixinLiveBarPlot

if pg.Qt.QT_LIB == pg.Qt.PYQT6:
    from PyQt6.QtCore import pyqtSlot
else:
    from PyQt5.QtCore import pyqtSlot


class LiveLinePlot(pg.PlotDataItem, MixinLivePlot, MixinLeadingLine):
    """Line plot"""

    @pyqtSlot()
    def update_leading_line(self):
        if self._vl_kwargs is not None:
            self._vl_kwargs["line"].setPos(self.xData[-1])

        if self._hl_kwargs is not None:
            self._hl_kwargs["line"].setPos(self.yData[-1])
        self.update_leading_text(self.xData[-1], self.yData[-1])


class LiveScatterPlot(pg.ScatterPlotItem, MixinLivePlot, MixinLeadingLine):
    """Scatter plot"""

    @pyqtSlot()
    def update_leading_line(self):
        last_point = self.data[-1]
        if self._vl_kwargs is not None:
            self._vl_kwargs["line"].setPos(last_point[0])
        if self._hl_kwargs is not None:
            self._hl_kwargs["line"].setPos(last_point[1])

        self.update_leading_text(last_point[0], last_point[1])


class LiveHBarPlot(pg.BarGraphItem, MixinLiveBarPlot, MixinLeadingLine):
    """Horizontal Bar Plot"""

    def __init__(self, x0=0, bar_height=1, **kwargs):
        self.bar_height = bar_height
        self.x0 = x0
        super().__init__(x0=x0, y=[0], width=0, height=0, **kwargs)

    def setData(self, x: float, y: float, kwargs: dict):
        self.setOpts(x0=self.x0, y=x, height=self.bar_height, width=y, **kwargs)
        self.sigPlotChanged.emit()

    @pyqtSlot()
    def update_leading_line(self):
        if self._vl_kwargs is not None:
            self._vl_kwargs["line"].setPos(self.opts["width"][-1])
        if self._hl_kwargs is not None:
            self._hl_kwargs["line"].setPos(self.opts["y"][-1])
        self.update_leading_text(self.opts["width"][-1], self.opts["y"][-1])


class LiveVBarPlot(pg.BarGraphItem, MixinLiveBarPlot, MixinLeadingLine):
    """Vertical Bar Plot"""

    def __init__(self, y0=0, bar_width=1, **kwargs):
        self.bar_width = bar_width
        self.y0 = y0
        super().__init__(y0=y0, x=[0], width=0, height=0, **kwargs)

    def setData(self, x: float, y: float, kwargs: dict):
        self.setOpts(y0=self.y0, x=x, height=y, width=self.bar_width, **kwargs)
        self.sigPlotChanged.emit()

    @pyqtSlot()
    def update_leading_line(self):
        if self._vl_kwargs is not None:
            self._vl_kwargs["line"].setPos(self.opts["x"][-1])
        if self._hl_kwargs is not None:
            self._hl_kwargs["line"].setPos(self.opts["height"][-1])
        self.update_leading_text(self.opts["x"][-1], self.opts["height"][-1])


def make_live(plot: GraphicsObject):
    """Convert plot into Live plot"""
    if isinstance(plot, pg.BarGraphItem):
        # Create horizontal bar plot in case of bar plot
        plot.bar_width = 1
        plot.y0 = 0
        plot.slot_new_data = lambda y, x, kwargs: plot.setOpts(y0=plot.y0, x=x, height=y, width=plot.bar_width,
                                                               **kwargs)
    elif isinstance(plot, pg.PlotCurveItem):
        plot.slot_new_data = lambda y, x, kwargs: plot.setData(np.array(x), np.array(y), **kwargs)
    else:
        plot.slot_new_data = lambda y, x, kwargs: plot.setData(x, y, **kwargs)



class Crosshair:
    """Keyword arguments related to Crosshair used in LivePlotWidget"""

    ENABLED = "Crosshair_Enabled"  # Toggle crosshair: bool
    LINE_PEN = "Crosshair_Line_Pen"  # Pen to draw crosshair: QPen
    TEXT_KWARGS = "Text_Kwargs"  # Kwargs for crosshair markers: dict


class Axis:
    TICK_FORMAT = "Tick_Format"  # "Tick format"
    DATETIME = "DateTime"
    TIME = "Time"


class LeadingLine:
    HORIZONTAL = "Horizontal"
    VERTICAL = "Vertical"
    TEXT_COLOR = "TextColor"
    AXIS_X = "x"
    AXIS_Y = "y"



import time
from collections import deque
from math import inf
from threading import Lock
from typing import List, Union

from pyqtgraph.Qt import QT_LIB, PYQT6

if QT_LIB == PYQT6:
    from PyQt6.QtCore import QObject, pyqtSignal
else:
    from PyQt5.QtCore import QObject, pyqtSignal

from pglive.sources.live_plot import MixinLivePlot, MixinLiveBarPlot, make_live


class DataConnector(QObject):
    sig_new_data = pyqtSignal(object, object, dict)
    sig_paused = pyqtSignal()
    sig_resumed = pyqtSignal()
    paused = False
    # Last update time, using perf_counter for most precise counter
    last_update = time.perf_counter()

    def __init__(self, plot: Union[MixinLivePlot, MixinLiveBarPlot], max_points=inf, update_rate=inf) -> None:
        """
        DataConnector is connecting plot with data and makes sure, that all updates are thread-safe.
        To make plot compatible and work with Connector, it must implement slot_new_data method.
        This can be achieved either by inheriting from LiveMixin or LiveMixinBarPlot class or DataConnector, tries to
        use make_live function and add slot in runtime.
        :param plot: Plot to be connected with Data
        :param max_points: Maximum amount of data points to plot
        :param float update_rate: Update rate in Hz
        """
        super().__init__()
        if not isinstance(plot, (MixinLivePlot, MixinLiveBarPlot)):
            # Attempt to convert plot into live if it's not already
            make_live(plot)
        # Data update lock, ensuring thread-safety
        self.data_lock = Lock()
        # Maximum number of points to plot
        self.max_points = max_points
        # Calculating update timeout from update_rate frequency
        self.update_timeout = 1 / update_rate
        if self.max_points == inf:
            # Use simple list if there is no point limits
            self.x, self.y = [], []
        else:
            # Use deque with maxlen otherwise
            self.x, self.y = deque(maxlen=self.max_points), deque(maxlen=self.max_points)
        # Set plot and connect sig_new_data with plot.slot_new_data
        self.plot = plot
        self.sig_new_data.connect(self.plot.slot_new_data)

    @property
    def max_points(self):
        return self._max_len

    @max_points.setter
    def max_points(self, new_max_len: Union[int, float]) -> None:
        assert new_max_len > 0
        self._max_len = new_max_len

    def pause(self) -> None:
        """Pause data plotting"""
        self.paused = True
        self.sig_paused.emit()

    def resume(self) -> None:
        """Resume data plotting"""
        self.paused = False
        self.sig_resumed.emit()

    def _skip_update(self) -> bool:
        """Skip update"""
        return self.paused or (time.perf_counter() - self.last_update) < self.update_timeout or self.data_lock.locked()

    def _update_data(self, **kwargs):
        """Update data and last update time"""
        # Notify all connected plots
        self.sig_new_data.emit(self.y, self.x, kwargs)
        self.last_update = time.perf_counter()

    def cb_set_data(self, y: List[Union[int, float]], x: List[Union[int, float]] = None, **kwargs) -> None:
        """Replace current data"""
        if self._skip_update():
            return

        with self.data_lock:
            if self.max_points == inf:
                self.y = y
            else:
                self.y = deque(y, maxlen=self.max_points)
            if x is not None:
                if self.max_points == inf:
                    self.x = x
                else:
                    self.x = deque(x, maxlen=self.max_points)
            else:
                self.x = range(len(self.y))
            self._update_data(**kwargs)

    def cb_append_data_point(self, y: Union[int, float], x: Union[int, float] = None, **kwargs) -> None:
        """Append new data point"""
        if self._skip_update():
            return

        with self.data_lock:
            self.y.append(y)
            if x is not None:
                self.x.append(x)
            elif len(self.x) == 0:
                self.x.append(0)
            else:
                self.x.append(self.x[-1] + 1)
            self._update_data(**kwargs)



import pglive.examples_pyqt6 as examples
import signal
import time
from math import sin
from threading import Thread
from time import sleep

import pyqtgraph as pg

from pglive.kwargs import Axis
from pglive.sources.data_connector import DataConnector
from pglive.sources.live_axis import LiveAxis
from pglive.sources.live_plot import LiveLinePlot
from pglive.sources.live_plot_widget import LivePlotWidget

"""
In this example Line plot is displayed.
"""
connectors = []

layout = pg.LayoutWidget()
# Define Time plot
left_axis = LiveAxis("left", axisPen="red", textPen="red")
bottom_axis = LiveAxis("bottom", axisPen="green", textPen="green", **{Axis.TICK_FORMAT: Axis.TIME})
time_axis_plot_widget = LivePlotWidget(title="Time Line Plot @ 100Hz",
                                       axisItems={'bottom': bottom_axis, 'left': left_axis})
plot = LiveLinePlot()
time_axis_plot_widget.addItem(plot)
connectors.append(DataConnector(plot, max_points=600))
layout.addWidget(time_axis_plot_widget)

# Define DateTime plot
left_axis = LiveAxis("left", axisPen="purple", textPen="purple")
bottom_axis = LiveAxis("bottom", axisPen="yellow", textPen="yellow", **{Axis.TICK_FORMAT: Axis.DATETIME})
datetime_axis_plot_widget = LivePlotWidget(title="DateTime Line Plot @ 100Hz",
                                           axisItems={'bottom': bottom_axis, 'left': left_axis})
plot = LiveLinePlot()
datetime_axis_plot_widget.addItem(plot)

layout.addWidget(datetime_axis_plot_widget)
connectors.append(DataConnector(plot, max_points=600))

layout.show()


def sin_wave_generator(*data_connectors):
    """Sinus wave generator"""
    x = 0
    while examples.running:
        x += 1
        for data_connector in data_connectors:
            data_connector.cb_append_data_point(sin(x * 0.01), time.time())
        sleep(0.01)


Thread(target=sin_wave_generator, args=connectors).start()
signal.signal(signal.SIGINT, lambda sig, frame: examples.stop())
examples.app.exec()
examples.stop()



import signal
import sys
from math import sin
from time import sleep

from PyQt6.QtGui import QColor
from PyQt6.QtWidgets import QApplication

running = True
app = QApplication(sys.argv)


def stop():
    """Stop current QApplication"""
    global running
    running = False
    app.exit(0)


def sin_wave_generator(*data_connectors, flip=False):
    """Sinus wave generator"""
    x = 0
    while running:
        x += 1
        for data_connector in data_connectors:
            if flip:
                data_connector.cb_append_data_point(x, sin(x * 0.025))
            else:
                data_connector.cb_append_data_point(sin(x * 0.025), x)
        sleep(0.01)


def colors():
    """Primitive color cycler"""
    while True:
        for r in range(50, 250, 2):
            for g in range(50, 250, 2):
                for b in range(50, 250, 2):
                    yield QColor(r, g, b)
                for c in range(50, 250, 2):
                    yield QColor(r, g, b - c)


# Initiate color generator
colors = colors()
# Connect SIGINT with stop function
signal.signal(signal.SIGINT, lambda sig, frame: stop())



import sys
from math import sin
from threading import Thread
from time import sleep

from PyQt6.QtWidgets import QApplication
from pglive.sources.live_plot import LiveLinePlot

from pglive.sources.data_connector import DataConnector
from pglive.sources.live_plot_widget import LivePlotWidget

"""
In this example Line plot is displayed.
"""
app = QApplication(sys.argv)
running = True

plot_widget = LivePlotWidget(title="Line Plot @ 100Hz")
plot_curve = LiveLinePlot()
plot_widget.addItem(plot_curve)
# DataConnector holding 600 points and plots @ 100Hz
data_connector = DataConnector(plot_curve, max_points=600, update_rate=100)


def sin_wave_generator(connector):
    """Sinus wave generator"""
    x = 0
    while running:
        x += 1
        data_point = sin(x * 0.01)
        # Callback to plot new data point
        connector.cb_append_data_point(data_point, x)

        sleep(0.01)


plot_widget.show()
Thread(target=sin_wave_generator, args=(data_connector,)).start()
app.exec()
running = False


from typing import Union, Optional

import pyqtgraph as pg

if pg.Qt.QT_LIB == pg.Qt.PYQT6:
    from PyQt6.QtCore import QPointF, pyqtSignal, QEvent
    from PyQt6.QtGui import QPen
else:
    from PyQt5.QtCore import QPointF, pyqtSignal, QEvent
    from PyQt5.QtGui import QPen

from pglive.kwargs import Crosshair


class LivePlotWidget(pg.PlotWidget):
    """Implements main plot widget for all live plots"""
    mouse_position: Optional[QPointF] = None
    sig_crosshair_moved = pyqtSignal(QPointF)
    sig_crosshair_out = pyqtSignal()
    sig_crosshair_in = pyqtSignal()

    def __init__(self, parent=None, background='default', plotItem=None, **kwargs) -> None:
        super().__init__(parent=parent, background=background, plotItem=plotItem, **kwargs)
        self.crosshair_enabled = kwargs.get(Crosshair.ENABLED, False)
        self.crosshair_items = []
        if self.crosshair_enabled:
            self._add_crosshair(kwargs.get(Crosshair.LINE_PEN, None),
                                kwargs.get(Crosshair.TEXT_KWARGS, {}))

        # Override addItem method
        def addItem(*args):
            if hasattr(args[0], "_vl_kwargs") and args[0]._vl_kwargs is not None:
                self.plotItem.addItem(args[0]._vl_kwargs["line"], ignoreBounds=True)
                self.plotItem.addItem(args[0]._vl_kwargs["text"], ignoreBounds=True)
            if hasattr(args[0], "_hl_kwargs") and args[0]._hl_kwargs is not None:
                self.plotItem.addItem(args[0]._hl_kwargs["line"], ignoreBounds=True)
                self.plotItem.addItem(args[0]._hl_kwargs["text"], ignoreBounds=True)
            self.plotItem.addItem(*args)

        self.addItem = addItem

    def _add_crosshair(self, crosshair_pen: QPen, crosshair_text_kwargs: dict) -> None:
        """Add crosshair into plot"""
        self.vLine = pg.InfiniteLine(angle=90, movable=False, pen=crosshair_pen)
        self.hLine = pg.InfiniteLine(angle=0, movable=False, pen=crosshair_pen)
        self.x_value_label = pg.TextItem(**crosshair_text_kwargs)
        self.y_value_label = pg.TextItem(**crosshair_text_kwargs)
        # All Crosshair items
        self.crosshair_items = [self.hLine, self.vLine, self.x_value_label, self.y_value_label]
        for item in self.crosshair_items:
            # Make sure, that every crosshair item is painted on top of everything
            item.setZValue(999)
            self.addItem(item, ignoreBounds=True)
        # Hide crosshair at the beginning
        self.hide_crosshair()

    def _update_crosshair_position(self) -> None:
        """Update position of crosshair based on mouse position"""
        if self.mouse_position:
            mouse_point = self.plotItem.vb.mapSceneToView(self.mouse_position)
            # Move crosshair to mouse pointer position
            self.vLine.setPos(mouse_point.x())
            self.hLine.setPos(mouse_point.y())

            self.x_value_label.setText(f"X = {self.x_format(mouse_point.x())}")
            self.y_value_label.setText(f"Y = {self.y_format(mouse_point.y())}")

            x_pos = self.mouse_position.x() + 5
            y_pos = self.mouse_position.y() - self.x_value_label.boundingRect().height()

            # Resolve position of crosshair text to be in view of the plot
            text_height = self.x_value_label.boundingRect().height()
            if self.mouse_position.x() + self.x_value_label.boundingRect().width() > self.plotItem.width():
                x_pos = self.mouse_position.x() - 5 - self.x_value_label.boundingRect().width()
            elif self.mouse_position.x() + self.y_value_label.boundingRect().width() > self.plotItem.width():
                x_pos = self.mouse_position.x() - 5 - self.x_value_label.boundingRect().width()
            if self.mouse_position.y() - self.x_value_label.boundingRect().height() - 2 * text_height - 5 < 0:
                y_pos += text_height * 2 + 5

            # Move X marker to position
            x_marker_point = self.plotItem.vb.mapSceneToView(
                QPointF(x_pos, y_pos - self.x_value_label.boundingRect().height()))
            self.x_value_label.setPos(x_marker_point.x(), x_marker_point.y())

            # Move Y marker to position
            y_marker_point = self.plotItem.vb.mapSceneToView(QPointF(x_pos, y_pos))
            self.y_value_label.setPos(y_marker_point.x(), y_marker_point.y())

            # Emit crosshair moved signal
            self.sig_crosshair_moved.emit(mouse_point)

    def x_format(self, value: Union[int, float]) -> str:
        """X tick format"""
        return str(round(value, 4))

    def y_format(self, value: Union[int, float]) -> str:
        """Y tick format"""
        return str(round(value, 4))

    def leaveEvent(self, ev: QEvent) -> None:
        """Mouse left PlotWidget"""
        if self.crosshair_enabled:
            self.hide_crosshair()
        self.mouse_position = None
        self.sig_crosshair_out.emit()
        super().leaveEvent(ev)

    def enterEvent(self, ev: QEvent) -> None:
        """Mouse enter PlotWidget"""
        if self.crosshair_enabled:
            self.show_crosshair()
        self.sig_crosshair_in.emit()
        super().enterEvent(ev)

    def mouseMoveEvent(self, ev: QEvent) -> None:
        """Mouse moved in PlotWidget"""
        if pg.Qt.QT_LIB == pg.Qt.PYQT6:
            ev_pos = ev.position()
        else:
            ev_pos = ev.pos()

        if self.crosshair_enabled and self.sceneBoundingRect().contains(ev_pos):
            self.mouse_position = ev_pos
            self._update_crosshair_position()
        super().mouseMoveEvent(ev)

    def paintEvent(self, ev: QEvent) -> None:
        """Update crosshair position when replot"""
        self._update_crosshair_position()
        return super().paintEvent(ev)

    def hide_crosshair(self) -> None:
        """Hide crosshair items"""
        for item in self.crosshair_items:
            item.hide()

    def show_crosshair(self) -> None:
        """Show crosshair items"""
        for item in self.crosshair_items:
            item.show()
